@@ -99,10 +99,6 @@ class PropertyMetaclass(type):
 
         # Ensure prop names are valid and overwrite properties with @property
         for key, prop in iteritems(prop_dict):
-            if key[0] == '_':
-                raise AttributeError(
-                    'Property names cannot be private: {}'.format(key)
-                )
             if isinstance(prop, basic.Renamed) and prop.new_name not in _props:
                 raise TypeError('Invalid new name for renamed property: '
                                 '{}'.format(prop.new_name))
@@ -124,15 +120,37 @@ class PropertyMetaclass(type):
         for key, handler in iteritems(observer_dict):
             classdict[key] = handler.func
 
+        # Determine if private properties should be documented or just public
+        _doc_private = False
+        for base in reversed(bases):
+            _doc_private = getattr(base, '_doc_private', _doc_private)
+        _doc_private = classdict.get('_doc_private', _doc_private)
+
+        if not isinstance(_doc_private, bool):
+            raise AttributeError('_doc_private must be a boolean')
+
+        if _doc_private:
+            documented_props = sorted(_props)
+        else:
+            documented_props = sorted(p for p in _props if p[0] != '_')
+
         # Order the properties for the docs (default is alphabetical)
-        _doc_order = classdict.pop('_doc_order', None)
+        _doc_order = None
+        for base in reversed(bases):
+            _doc_order = getattr(base, '_doc_order', _doc_order)
+            if (
+                    not isinstance(_doc_order, (list, tuple)) or
+                    sorted(list(_doc_order)) != documented_props
+            ):
+                _doc_order = None
+        _doc_order = classdict.get('_doc_order', _doc_order)
         if _doc_order is None:
-            _doc_order = sorted(_props)
+            _doc_order = documented_props
         elif not isinstance(_doc_order, (list, tuple)):
             raise AttributeError(
                 '_doc_order must be a list of property names'
             )
-        elif sorted(list(_doc_order)) != sorted(_props):
+        elif sorted(list(_doc_order)) != documented_props:
             raise AttributeError(
                 '_doc_order must be unspecified or contain ALL property names'
             )
@@ -140,11 +158,13 @@ class PropertyMetaclass(type):
         # Sort props into required, optional, and immutable
         doc_str = classdict.get('__doc__', '')
         req = [key for key in _doc_order
-               if getattr(_props[key], 'required', False)]
+               if key[0] != '_' and getattr(_props[key], 'required', False)]
         opt = [key for key in _doc_order
-               if not getattr(_props[key], 'required', True)]
+               if key[0] != '_' and not getattr(_props[key], 'required', True)]
         imm = [key for key in _doc_order
-               if not hasattr(_props[key], 'required')]
+               if key[0] != '_' and not hasattr(_props[key], 'required')]
+        priv = [key for key in _doc_order
+                if key[0] == '_']
 
         # Build the documentation based on above sorting
         if req:
@@ -159,6 +179,10 @@ class PropertyMetaclass(type):
             doc_str += '\n\n**Other Properties:**\n\n' + '\n'.join(
                 ('* ' + _props[key].sphinx() for key in imm)
             )
+        if priv:
+            doc_str += '\n\n**Private Properties:**\n\n' + '\n'.join(
+                ('* ' + _props[key].sphinx() for key in priv)
+            )
         classdict['__doc__'] = doc_str
 
         # Create the new class
@@ -202,8 +226,8 @@ class PropertyMetaclass(type):
         """
 
         obj = cls.__new__(cls, *args, **kwargs)
-        obj._backend = dict()
-        obj._listeners = dict()
+        object.__setattr__(obj, '_backend', dict())
+        object.__setattr__(obj, '_listeners', dict())
 
         # Register the listeners
         for _, val in iteritems(obj._prop_observers):
@@ -253,9 +277,13 @@ class HasProperties(with_metaclass(PropertyMetaclass, object)):
     def __init__(self, **kwargs):
         # Set the keyword arguments with change notifications
         for key, val in iteritems(kwargs):
-            if not hasattr(self, key) and key not in self._props.keys():
+            prop = self._props.get(key, None)
+            if not prop and not hasattr(self, key):
                 raise AttributeError("Keyword input '{}' is not a known "
                                      "property or attribute".format(key))
+            if isinstance(prop, basic.DynamicProperty):
+                raise AttributeError("Dynamic property '{} cannot be set on "
+                                     "init".format(key))
             setattr(self, key, val)
 
     def _get(self, name):
@@ -311,16 +339,23 @@ class HasProperties(with_metaclass(PropertyMetaclass, object)):
             val = val()
         setattr(self, name, val)
 
-    @utils.stop_recursion_with(True)
     def validate(self):
         """Call all registered class validator methods
 
         These are all methods decorated with :code:`@properties.validator`.
         Validator methods are expected to raise an error if they fail.
         """
-        for val in itervalues(self._class_validators):
-            val.func(self)
-        return True
+        if getattr(self, '_getting_validated', False):
+            return True
+        self._getting_validated = True
+        try:
+            for val in itervalues(self._class_validators):
+                valid = val.func(self)
+                if valid is False:
+                    raise ValueError('Validation failed')
+            return True
+        finally:
+            self._getting_validated = False
 
     @handlers.validator
     def _validate_props(self):
@@ -335,13 +370,13 @@ class HasProperties(with_metaclass(PropertyMetaclass, object)):
                     raise ValueError(
                         'Invalid value for property {}: {}'.format(key, value)
                     )
-            prop.assert_valid(self)
+            if not prop.assert_valid(self):
+                raise ValueError(
+                    'Invalid value for property {}: {}'.format(key, value)
+                )
         return True
 
-    @utils.stop_recursion_with(
-        utils.SelfReferenceError('Object contains unserializable self reference')
-    )
-    def serialize(self, include_class=True, **kwargs):
+    def serialize(self, include_class=True, save_dynamic=False, **kwargs):
         """Serializes a **HasProperties** instance to dictionary
 
         This uses the Property serializers to serialize all Property values
@@ -354,18 +389,34 @@ class HasProperties(with_metaclass(PropertyMetaclass, object)):
         * **include_class** - If True (the default), the name of the class
           will also be saved to the serialized dictionary under key
           :code:`'__class__'`
+        * **save_dynamic** - If True, dynamic properties are written to
+          the serialized dict (default: False).
         * Any other keyword arguments will be passed through to the Property
           serializers.
         """
-        data = (
-            (k, v.serialize(
-                self._get(v.name), include_class=include_class, **kwargs
-            )) for k, v in iteritems(self._props)
-        )
-        json_dict = {k: v for k, v in data if v is not None}
-        if include_class:
-            json_dict.update({'__class__': self.__class__.__name__})
-        return json_dict
+        if getattr(self, '_getting_serialized', False):
+            raise utils.SelfReferenceError('Object contains unserializable '
+                                           'self reference')
+        self._getting_serialized = True
+        try:
+            kwargs.update({
+                'include_class': include_class,
+                'save_dynamic': save_dynamic
+            })
+            if save_dynamic:
+                prop_source = self._props
+            else:
+                prop_source = self._backend
+            data = (
+                (key, self._props[key].serialize(getattr(self, key), **kwargs))
+                for key in prop_source
+            )
+            json_dict = {k: v for k, v in data if v is not None}
+            if include_class:
+                json_dict.update({'__class__': self.__class__.__name__})
+            return json_dict
+        finally:
+            self._getting_serialized = False
 
     @classmethod
     def deserialize(cls, value, trusted=False, verbose=True, **kwargs):
@@ -403,6 +454,7 @@ class HasProperties(with_metaclass(PropertyMetaclass, object)):
                         rcl=value['__class__'], cl=cls.__name__
                     ), RuntimeWarning
                 )
+        kwargs.update({'trusted': trusted, 'verbose': verbose})
         state, unused = utils.filter_props(cls, value, True)
         unused.pop('__class__', None)
         if unused and verbose:
@@ -411,9 +463,7 @@ class HasProperties(with_metaclass(PropertyMetaclass, object)):
             ), RuntimeWarning)
         newstate = {}
         for key, val in iteritems(state):
-            newstate[key] = cls._props[key].deserialize(
-                val, trusted=trusted, **kwargs
-            )
+            newstate[key] = cls._props[key].deserialize(val, **kwargs)
         mutable, immutable = utils.filter_props(cls, newstate, False)
         with handlers.listeners_disabled():
             newinst = cls(**mutable)
@@ -427,28 +477,32 @@ class HasProperties(with_metaclass(PropertyMetaclass, object)):
             valid_val = self._props[key].validate(self, pickle.loads(val))
             self._backend[key] = valid_val
 
-    @utils.stop_recursion_with(
-        utils.SelfReferenceError('Object contains unpicklable self reference')
-    )
     def __reduce__(self):
-        data = ((k, self._get(v.name)) for k, v in iteritems(self._props))
-        pickle_dict = {k: pickle.dumps(v) for k, v in data if v is not None}
-        return (self.__class__, (), pickle_dict)
+        if getattr(self, '_getting_pickled', False):
+            raise utils.SelfReferenceError('Object contains unpicklable self '
+                                           'reference')
+        self._getting_pickled = True
+        try:
+            data = ((k, self._get(v.name)) for k, v in iteritems(self._props))
+            pickle_dict = {
+                k: pickle.dumps(v) for k, v in data if v is not None
+            }
+            return (self.__class__, (), pickle_dict)
+        finally:
+            self._getting_pickled = False
 
-    @utils.stop_recursion_with(False)
     def equal(self, other):
         """Determine if two **HasProperties** instances are equivalent
 
         Equivalence is determined by checking if all Property values on
         two instances are equal, using :code:`Property.equal`.
         """
-        warn('HasProperties.equal has been depricated in favor of '
+        warn('HasProperties.equal has been deprecated in favor of '
              'properties.equal and will be removed in the next release',
              FutureWarning)
         return equal(self, other)
 
 
-@utils.stop_recursion_with(False)
 def equal(value_a, value_b):
     """Determine if two **HasProperties** instances are equivalent
 
@@ -470,22 +524,29 @@ def equal(value_a, value_b):
             not isinstance(value_b, HasProperties)
     ):
         return value_a == value_b
-    if value_a is value_b:
-        return True
-    if value_a.__class__ is not value_b.__class__:
+    if getattr(value_a, '_testing_equality', False):
         return False
-    for prop in itervalues(value_a._props):
-        prop_a = getattr(value_a, prop.name)
-        prop_b = getattr(value_b, prop.name)
-        if prop_a is None and prop_b is None:
-            continue
-        if prop_a is None or prop_b is None:
+    value_a._testing_equality = True                                           #pylint: disable=protected-access
+    try:
+        if value_a is value_b:
+            return True
+        if value_a.__class__ is not value_b.__class__:
             return False
-        if prop.equal(getattr(value_a, prop.name),
-                      getattr(value_b, prop.name)):
-            continue
-        return False
-    return True
+        for prop in itervalues(value_a._props):
+            prop_a = getattr(value_a, prop.name)
+            prop_b = getattr(value_b, prop.name)
+            if prop_a is None and prop_b is None:
+                continue
+            if (
+                    prop_a is not None and
+                    prop_b is not None and
+                    prop.equal(prop_a, prop_b)
+            ):
+                continue
+            return False
+        return True
+    finally:
+        value_a._testing_equality = False                                      #pylint: disable=protected-access
 
 
 def copy(value, **kwargs):
@@ -501,4 +562,6 @@ def copy(value, **kwargs):
     if not isinstance(value, HasProperties):
         raise ValueError('properties.copy may only be used to copy'
                          'HasProperties instances')
+    kwargs.update({'include_class': kwargs.get('include_class', True)})
+    kwargs.update({'trusted': kwargs.get('trusted', True)})
     return value.__class__.deserialize(value.serialize(**kwargs), **kwargs)
